//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

function dateFormat(date) {
    var datetime = date.getUTCFullYear() + "-";
    var month = String(date.getUTCMonth() + 1);
    datetime += (month.length == 1 ?  '0' + month : month) + "-";
    var day = date.getUTCDate();
    datetime += (day < 10 ? '0' + day : day);

    datetime += 'T';

    var hour = date.getUTCHours();
    datetime += (hour < 10 ? '0' + hour : hour) + ':';
    var minutes = date.getUTCMinutes();
    datetime += (minutes < 10 ? '0' + minutes : minutes) + ':';
    var seconds = date.getUTCSeconds();
    datetime += (seconds < 10 ? '0' + seconds : seconds) + "Z";

    return datetime;
}

function JSAtomAPI() {
}

JSAtomAPI.prototype= {
    _req: null,
    _url: null,

    ERROR_HTTP:   10,
    ERROR_FAULT:   11,
    ERROR_PARSER:   12,

    init: function(url) {
        this._url = url;
    },

    doRequest: function(listener, method, url, body, processor) {
        var inst = this;
        this._req = new XMLHttpRequest();
        this._req.onreadystatechange = function (aEvt) {
            if(inst._req.readyState == 4) {
                //dump("\nRESPONSE\n" + inst._req.responseText);
                FlockLogger.write("atomapi.txt", "\nRECEIVING\n");
                FlockLogger.write("atomapi.txt", inst._req.responseText);
		try {
			if(inst._req.status == 200 || inst._req.status == 201)
			{
			    try
			    {
				processor(listener, inst);
			    }
			    catch(e)
			    {
				listener.onError(inst.ERROR_PARSER);
			    }
			}
			else {
				var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
				var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
				var stringBundle = stringBundleService.createBundle("chrome://flock/locale/blog/blog.properties", localeService.getApplicationLocale());
				var fault =  {
					faultString: stringBundle.GetStringFromName("flock.blog.atom.error"),
				};
					fault.faultString += inst._req.responseText;
					listener.onFault(fault);
			}
		} catch(e) {
                        listener.onError(inst.ERROR_PARSER);
		}
            }
        };
        rval = this._req.open(method, url, true);
        //dump("\n\n\n\n\nSending: " + method + "\n\n" + url + "\n>>>\n" + body + "\n\n\n\n");
        FlockLogger.write("atomapi.txt", "\nSENDING to " + url + "\n");
        FlockLogger.write("atomapi.txt", body);
        rval = this._req.send(body); 
    },

    getRecentPosts: function(listener) {
        var url = this._url; 
        url.match(/(.+\/)(.+)/);
        if(RegExp.$2=="post") {
            url = RegExp.$1 + "feed";
        }
        this.doRequest(listener,"GET", url,null, this.parseRecentPosts);
    },
    getUsersBlogs: function(listener, url) {
        var url = this._url.match(/(.+)\//);
        url = RegExp.$1;
        this.doRequest(listener,"GET", url, null, this.parseUsersBlogs);
    },

    parseUsersBlogs: function(listener, inst) {
        try {
            var rval = new Array();
            var dom = inst._req.responseXML;
            var links = dom.getElementsByTagName("link");
            for(var i=0;i<links.length;++i)
            {
                var link = links[i];
                var title = link.getAttribute("title");
                var rel = link.getAttribute("rel");
                var href = link.getAttribute("href");
                if(rel!="service.post") continue;

                href.match(/.+\/(.+)/);
                var bid = RegExp.$1;

                var obj  =
                {
                    blogName: title,
                    blogid:  bid,
                    url:  href,
                };
                rval[rval.length++] = obj;
            }
            listener.onResult(rval);
        }
        catch(e) {
            //alert(e + " " + e.lineNumber);
        }
    },

/*
    parseSinglePost: function(listener, inst) {
    },
    */

    parseRecentPosts: function(listener, inst) {
        var rval = new Array();
        var dom = inst._req.responseXML;
	    var entries = dom.getElementsByTagName("entry");
        for(var i=0;i<entries.length;++i)
        {
            try
            {
                var entry_n = entries[i];
                var content_n = inst.getNamedChild(entry_n, "content");
                var title_n = inst.getNamedChild(entry_n, "title");
                var created_n = inst.getNamedChild(entry_n, "issued");
                var author_n = inst.getNamedChild(entry_n, "author");
                var link_n = null;
                var permalink_n = null;
                var atomid_n = inst.getNamedChild(entry_n, "id");
                var atomid = "";
                if(atomid_n) atomid = atomid_n.firstChild.nodeValue;

                for(var j=0;j<entry_n.childNodes.length;++j) {
                    if(entry_n.childNodes[j].nodeName=="link") {
                        var tmp = entry_n.childNodes[j];
                        if(tmp.getAttribute("rel").match(/edit/)) {
                            link_n = tmp;
                        }
                        if(tmp.getAttribute("rel").match(/alternate/)) {
                            permalink_n = tmp;
                        }
                    }
                }

                var permaLink = permalink_n.getAttribute("href");
                var href = link_n.getAttribute("href");
                href.match(/.+\/(.+)/);
                var postid = RegExp.$1;

                var dateval = created_n.firstChild.nodeValue;
                var date = flock_parseDate(dateval);

                var xs = new XMLSerializer();

                var data = "";
                if(content_n) {
                    for(var j=0;j<content_n.childNodes.length;++j) {
                        data += xs.serializeToString(content_n.childNodes.item(j));
                    }
                }
                data = data.replace(/&lt;/g,"<");
                data = data.replace(/&gt;/g,">");
                data = data.replace(/&quot;/g,"\"");
                data = data.replace(/&amp;/g,"&");

                var obj  =
                {
                    description: data,
                    title: title_n.firstChild.nodeValue,
                    dateCreated: date,
                    postid:  postid,
                    editURI:  href,
                    permaLink:  permaLink,
                    atomid:  atomid,
                    
                };
                rval[rval.length] = obj;
            }
            catch(e)
            {
                alert(e);
            }
        }
        listener.onResult(rval);
    },
    getNamedChild: function(node, name) {
        for(var i=0;i<node.childNodes.length;++i) {
            if(node.childNodes[i].nodeName==name) return node.childNodes[i];
        }
        return null;
    },
    deletePost: function (listener, postid, aEditURI) {
        var url = this._url;
        if(postid) url += "/" + postid;
        if(aEditURI) url = aEditURI;
        this.doRequest(listener,"DELETE", url,null, this.handleDelete);
    },
    getPost: function (listener, postid, aEditURI) {
        var url = this._url;
        if(postid) url += "/" + postid;
        var myListener = {
            onResult: function(aResult) {
                var rval = aResult[0];
                listener.onResult(rval);
            },
            onError: function(aError) {
                listener.onError(aError);
            },
            onFault: function(aFault) {
                listener.onFault(aFault);
            },
        };
        this.doRequest(myListener,"GET", url,null, this.parseRecentPosts);
    },
    doPost: function (listener, method, postid, title, description, date_created, public, aEditURI, aAtomID) {

        description = description.replace(/\r/g,"");
        description = description.replace(/\n/g," ");
        var url = this._url;
        if(postid) url += "/" + postid;

        if(aEditURI) url = aEditURI;

        var date = dateFormat(date_created);
        var body = "";
        var version = "Flock Alpha";
        body += '<?xml version="1.0"?>';
        body += '<entry xmlns="http://purl.org/atom/ns#">';

        var draft = "false";
        if(!public) draft = "true";

        body += '<draft xmlns="http://purl.org/atom-blog/ns#">' + draft + '</draft> '
        body += '<title mode="escaped" type="text/plain">' + title + '</title>';
        body += '<issued>' + date + '</issued>';
        if(aAtomID && !url.match(/blogger.com/)) body += '<id>' + aAtomID + '</id>';
        body += '<generator url="http://flock.com">' + version + '</generator>';
        body += '<content type="application/xhtml+xml">';
        //description = description.replace(/&\S+?;/g, "");
        description = description.replace(/&nbsp;/g, "");
        var test = description;
        test = test.replace(/\s/g,"");
        if(!test.length) description = "<br/>";
        body += '<div xmlns="http://www.w3.org/1999/xhtml">' + description + '</div>';
//        body += description;
        body += '</content> </entry>';
        dump(body);
        this.doRequest(listener,method,url,body,this.parsePosts,public);
    },
    editPost: function (listener, postid, title, description, date_created, public, aEditURI, aAtomID) {
        this.doPost(listener, "PUT", postid, title, description, date_created,public, aEditURI, aAtomID);
    },
    newPost: function (listener, title, description, date_created,public) {
        this.doPost(listener, "POST", null, title, description, date_created,public);
    },
    parsePosts: function(listener, inst) {
        var dom = inst._req.responseXML;
        if(!dom)
        {
            // LiveJournal - everone wants to be different, don't they?
            if(inst._req.responseText.match(/OK/)) {
                listener.onResult(1);
                return;
            }
            else {
                var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
                var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
                var stringBundle = stringBundleService.createBundle("chrome://flock/locale/blog/blog.properties", localeService.getApplicationLocale());
                var fault =  {
                    faultString: stringBundle.GetStringFromName("flock.blog.atom.error"),
                };
                listener.onFault(fault);
                return;
            }
        }
	    var entries = dom.getElementsByTagName("entry");
        var rval = new Object();
        try
        {
            var entry_n = entries[0];
            var link_n = null;
            var perm_link_n = null;
            var atomid_n = inst.getNamedChild(entry_n, "id");
            var atomid = "";
            if(atomid_n) atomid = atomid_n.firstChild.nodeValue;
            for(var j=0;j<entry_n.childNodes.length;++j) {
                if(entry_n.childNodes[j].nodeName=="link") {
                    var tmp = entry_n.childNodes[j];
                    if(tmp.getAttribute("rel").match(/edit/)) {
                        link_n = tmp;
                    }
                    else if(tmp.getAttribute("rel").match(/alternate/)) {
                        perm_link_n = tmp;
                    }
                }
            }
            var href = link_n.getAttribute("href");
            rval.editURI = href;
            rval.atomid = atomid;
            href.match(/.+\/(.+)/);
            rval.uid = RegExp.$1;;
            var perm_link = perm_link_n.getAttribute("href");
            //Livejournal hackage
            if(rval.editURI.length>0 && rval.atomid.length==0) {
                if(perm_link.match(/livejournal/)) {
                    perm_link.match(/http.\/\/(\w+?)\.livejournal.com.*\/(\w+)\.html/);
                    rval.atomid = "urn:lj:livejournal.com:atom1:" + RegExp.$1 + ":" + RegExp.$2;
                }
            }
            rval.permaLink = perm_link;
        }
        catch(e)
        {
            alert(e);
        }
        listener.onResult(rval);
    },
    handleDelete: function(listener, inst) {
        listener.onResult(1);
    },
}
