var photoBrowser = {};

/////////////////////////////
// constants
/////////////////////////////
const LIST_WATCHED = Components.interfaces.flockIPhotoPeopleService.LIST_WATCHED;
const LIST_RECENT = Components.interfaces.flockIPhotoPeopleService.LIST_RECENT;
const LIST_ACCOUNTS = Components.interfaces.flockIPhotoPeopleService.LIST_ACCOUNTS;

const VIEW_SINGLE = 1;
const VIEW_MULTI = 2;

/////////////////////////////
// member vars
/////////////////////////////
photoBrowser.mLastLoad = null;
photoBrowser.mCurrentPerson = null;
photoBrowser.mCurrentSearch = null;
photoBrowser.mSize = "thumb";

/////////////////////////////
// UI
/////////////////////////////
photoBrowser.APIMGR = Components.classes['@flock.com/photo-api-manager;1?'] .getService(Components.interfaces.flockIPhotoAPIManager);
photoBrowser.PPSVC = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-people-photo'].getService(Components.interfaces.flockIPhotoPeopleService);

function topbarUnload() {
    photoBrowser.unload();
}

/////////////////////////////
// initialization
/////////////////////////////
photoBrowser.peopleServiceListener = {
    onNewPhotosTouched: function() {
        photoBrowser.setupMenulists();
    },
    onNewPhotos: function() {
        photoBrowser.setupMenulists();
    },
    onUpdate: function() {
        photoBrowser.setupMenulists();
    }
}

photoBrowser.apiListener = {
    cache: {},
    onAPIStateChange: function(aAPI) {
        var changed = true;
        if(this.cache[aAPI.shortName] == aAPI.authState) changed = false;
        this.cache[aAPI.shortName] = aAPI.authState;
        if(aAPI.isLoggedIn && changed) {
            photoBrowser.loadPersonPre(aAPI.getAuthPerson());
        }
        else {
            this.cache[aAPI.shortName] = false;
        }
        photoBrowser.updateLoginUI(aAPI);
    },
    onError: function(aError) {
        var loginButton = $('flock-photo-browser-login');
        loginButton.hidden = false; 
        loginButton.setAttribute('label', 'Login')
        loginButton.disabled = false;
    },
}

photoBrowser.setupPersonMenu = function(popup, event) {
 
  var kids = popup.childNodes;
  for (var i=kids.length-1; i>=0; i--) {
    if (!(kids[i].getAttribute('dontDelete')))
      popup.removeChild(kids[i]);
  }

  var svcs = photoBrowser.APIMGR.services;
  while(svcs.hasMoreElements()) {
    var svc = svcs.getNext();
    var serviceName = svc.serviceName; if(!serviceName) continue;//tmp workaround
    if(svc.isLoggedIn) {
      var person = svc.getAuthPerson();
      var account = document.createElement("menuitem");
      popup.insertBefore( account, popup.firstChild );
      account.label = person.username;
      account.setAttribute("image", svc.iconUrl);
      account.setAttribute("username", person.username);
      var shortName = svc.shortName;
      account.setAttribute('class','menuitem-iconic');
      account.setAttribute("shortName", shortName);
      account.setAttribute("oncommand", 
                "photoBrowser.loadPersonPre(photoBrowser.APIMGR.getAPIFromShortname('"+ shortName + "').getAuthPerson());");
      dump('adding ' +  account.getAttribute('shortName') + '\n');
    }
  }
};

photoBrowser.setupAccountMenu = function(aElm) {
    try {

    var kids = aElm.childNodes;
    var kidsToDelete = Array();
    for(var i=0;i<kids.length;++i) {
        var kid = kids.item(i);
        var dontDelete = kid.getAttribute("dontDelete");
        if(!dontDelete) {
            //aElm.removeChild(kid);
            kidsToDelete.push(kid);
        }
    }
        
    for (var j=0; j < kidsToDelete.length; j++) {
        aElm.removeChild(kidsToDelete[j]);
    }
    
    var count = 0;
    var svcs = photoBrowser.APIMGR.services;
    while(svcs.hasMoreElements()) {
        var svc = svcs.getNext();
        var serviceName = svc.serviceName; if(!serviceName) continue;//tmp workaround
        if(svc.isLoggedIn) {
            ++count;
            var person = svc.getAuthPerson();
            var account = document.createElement("menuitem");
            aElm.appendChild(account);
            account.label = person.username;
            account.setAttribute("image", svc.iconUrl);
            account.setAttribute("username", person.username);
            var shortName = svc.shortName;
            account.setAttribute('class','menuitem-iconic');
            account.setAttribute("shortName", shortName);
            account.setAttribute("oncommand", 
                "photoBrowser.loadPersonPre(photoBrowser.APIMGR.getAPIFromShortname('"+ shortName + "').getAuthPerson());");
           dump('adding ' +  account.getAttribute('shortName') + '\n');
        }
    }
    $("flock-photo-browse-accounts-empty").setAttribute("collapsed", (count!=0));
    aElm.appendChild($("flock-photo-browse-accounts-separator")); 
    aElm.appendChild($("flock-photo-browse-accounts-setup"));

    } catch(e) {
        alert(e);
    }
}

photoBrowser.clearNew = function() {
    window.setTimeout("photoBrowser.PPSVC.clearAllHasNewPhotos();",0);
}

photoBrowser.clearRecent = function() {
    window.setTimeout("photoBrowser.PPSVC.clearRecent();",0);
}

photoBrowser.setupNewMenu = function(aElm) {
    $("flock-photo-browse-new-find").setAttribute("disabled", (this.getFriendCount()==0));
    if(this.mNewCount==0) {
        $("flock-photo-browse-new-clear-sep").setAttribute("collapsed", true); 
    }
    else {
        $("flock-photo-browse-new-clear-sep").removeAttribute("collapsed");
        aElm.appendChild($("flock-photo-browse-new-clear-sep")); 
        aElm.appendChild($("flock-photo-browse-new-clear")); 
    }
    return;
}

photoBrowser.setupRecentMenu = function(aElm) {
    aElm.builder.rebuild();
    aElm.appendChild($("flock-photo-browse-recent-separator")); 
    aElm.appendChild($("flock-photo-browse-recent-clear"));
    $("cmd_photoPersonClearRecent").setAttribute("disabled", (this.getRecentCount() == 0));
}

photoBrowser.setupMenulist = function(aName, aCount) {
    var button = "flock-photo-browse-" + aName;
    var menu = "flock-photo-browse-" + aName + "-menu";
    var empty = "flock-photo-browse-" + aName + "-empty";
    if($(empty)) $(empty).setAttribute("collapsed", (aCount!=0));
    $(button).label = $(button).getAttribute("localizedName") + " (" + aCount + ")";
}

photoBrowser.rebuildMenulists = function() {
    $("flock-photo-browse-friends-menu").builder.rebuild();
    $("flock-photo-browse-new-menu").builder.rebuild();
    $("flock-photo-browse-recent-menu").builder.rebuild();
}

photoBrowser.getRecentCount = function() {
    return $("flock-photo-browse-recent-menu").childNodes.length - 4;
}

photoBrowser.getFriendCount = function() {
    return $("flock-photo-browse-friends-menu").childNodes.length - 2;
}

photoBrowser.setupMenulists = function() {
    var count;
    var isLoggedIn;
    
    if (this.APIMGR.getDefaultService())
        isLoggedIn = this.APIMGR.getDefaultService().isLoggedIn;
    else
        isLoggedIn = false;
        
    count = $("flock-photo-browse-friends-menu").childNodes.length - 3;
    $('flock-photo-browser-login').hidden = isLoggedIn;
    if (count == 0) {
      $("flock-photo-browse-friends").setAttribute('hidden', 'true');
      $('flock-photo-browse-instructions').hidden = !isLoggedIn;    
    } else {
      $("flock-photo-browse-instructions").hidden = true;
      $("flock-photo-browse-friends").removeAttribute('hidden');
      this.setupMenulist("friends", count);
    }

    count = 0;
    var kids = $("flock-photo-browse-new-menu").childNodes;
    for(var i=0;i<kids.length;++i) {
        var child = kids.item(i);
        if(child.nodeName!="menuitem") continue;
        if((child.getAttribute("hasNew")=="true")) {
            ++count;
            child.removeAttribute("collapsed");
        }
        else if((child.getAttribute("hasNew")=="false")) {
            child.setAttribute("collapsed", true);
        }
        else {
        }
    }

    if (count == 0) {
      $("flock-photo-browse-new").setAttribute('hidden', 'true');
    } else {
      $("flock-photo-browse-new").removeAttribute('hidden');
      this.setupMenulist("new", count);
    }
    $("cmd_photoPersonClearNew").setAttribute("disabled", (count == 0));
    $("cmd_photoPersonViewAllNew").setAttribute("disabled", (count == 0));
    this.mNewCount = count;
}

photoBrowser.loadInitialPerson = function() {
    var personArgument = null;
    try {
        personArgument = window.arguments[0];
    }
    catch(e) {
    }
    if(personArgument) {
        photoBrowser.loadPersonPre(personArgument);
    }
    else {
        var defaultServiceName = flock.prefs.getCharPref("flock.photo.defaultService");
        if(!defaultServiceName) {
            var svc = this.APIMGR.services.getNext();
            if (svc)
                defaultServiceName = svc.shortName;
            else
                throw new Error("No photo services registered in the api manager!");
        }
        var defaultService = photoBrowser.APIMGR.getAPIFromShortname(defaultServiceName);
        var defaultPerson = defaultService.firstRunPerson;
        var recentPerson = this.PPSVC.getMostRecentPhotoPerson();
        if(this.PPSVC.hasNew) {
            photoBrowser.loadAllNewPre();
        }
        else {
            photoBrowser.loadPersonPre( (recentPerson ? recentPerson : defaultPerson) );
        }
    }
}

photoBrowser.load = function() {
    try {
        this.PPSVC.addListener(this.peopleServiceListener);
        var date = new Date();
        this.initEveryoneUI();
        this.loadDefaults();
        this.loadInitialPerson();
        this.initFindUserUI();
        $("flock-photo-browse-slideybox").setListener(this.slideyBoxListener);
        photoBrowser.rebuildMenulists();
        photoBrowser.setupMenulists();
        this.APIMGR.addListener(this.apiListener);
        $$("flock_TopbarUIExtensionArea").hidden = true;

        this.mResizeListener = function(aEvent) {
            photoBrowser.refreshSlideyboxSize();
        }
        top.addEventListener("resize", this.mResizeListener, false);
    } catch(e) {
        alert(e);
    }
}

photoBrowser.loadDefaults = function() {
    var size = flock.prefs.getCharPref("flock.photo.photoSize");
    if(!size) size = "thumb";
    this.mSize = size;
    if (this.mSize == 'thumb') {
        $("flock-photo-browse-size").setAttribute('thumb', 'true');
    } else {
        $("flock-photo-browse-size").removeAttribute('thumb');
    }
}

photoBrowser.showEveryoneOn = function(aName) {
    var service = photoBrowser.APIMGR.getAPIFromShortname(aName);
    var name = $("flock-photo-browser-everyone").getAttribute("everyoneName");
    var everyone = {
        service: service,
        username: name,
        fullname: name,
        id: "",
    }
    this.loadPersonPre(everyone);
}

photoBrowser.initFindUserUI = function() {
    var popup = $("flock-photo-browse-service-menu-popup");
    while(popup.childNodes.length) {
        popup.removeChild(popup.firstChild);
    }
    var svcs = photoBrowser.APIMGR.services;
    while(svcs.hasMoreElements()) {
        var svc = svcs.getNext();
        var serviceName = svc.serviceName; if(!serviceName) continue;//tmp workaround
        var account = document.createElement("menuitem");
        popup.appendChild(account);
        account.label = svc.serviceName;
        account.value = svc.shortName;
    }
    if(this.mCurrentPerson) $("flock-photo-browse-service-menu").value = this.mCurrentPerson.service.shortName;
    $("flock-photo-browse-find-user-msg").value = $("flock-photo-browse-find-user-msg").getAttribute("initText");
}

photoBrowser.initEveryoneUI = function() {
    var svcs = photoBrowser.APIMGR.services;
    while(svcs.hasMoreElements()) {
        var svc = svcs.getNext();
        var serviceName = svc.serviceName; if(!serviceName) continue;//tmp workaround
        var everyone = document.createElement("menuitem");
        everyone.setAttribute("label", svc.serviceName);
        everyone.setAttribute("class", "menuitem-iconic");
        everyone.setAttribute("image", svc.iconUrl);
        everyone.value = svc.shortName;
        $("flock-photo-browser-everyone").appendChild(everyone);
        everyone.setAttribute("oncommand", "photoBrowser.showEveryoneOn('" + svc.shortName + "');");
    }
}

photoBrowser.unload = function() {
    this.PPSVC.removeListener(this.peopleServiceListener);
    this.APIMGR.removeListener(this.apiListener);
    top.removeEventListener("resize", this.mResizeListener, false);
    $$("flock_TopbarUIExtensionArea").hidden = false;
    if(this.mCurrentSearch) this.mCurrentSearch.invalid = true;
}

/////////////////////////////
// event handlers
/////////////////////////////

photoBrowser.refreshView = function() {
    switch(this.mViewType) {
        case VIEW_SINGLE:
            this.loadPersonPre(this.mCurrentPerson);
        break;
        case VIEW_MULTI:
            this.loadAllNewPre();
        break;
    }
}

photoBrowser.onSearchChange = function(aEvent) {
    if(aEvent.keyCode==aEvent.DOM_VK_RETURN) {
        this.refreshView();
    }
    if (aEvent.target.value.length != "")
        $('flock-photo-browser-search-container').setAttribute('hasSearchTerm', true);
    else
       $('flock-photo-browser-search-container').removeAttribute('hasSearchTerm');
}

photoBrowser.toggleSize = function(aElement, aEvent) {
    var thumb = $("flock-photo-browse-size").getAttribute('thumb') == 'true';
    if (thumb) {
        $("flock-photo-browse-size").removeAttribute('thumb');
        this.mSize = 'medium';
    } else {
        $("flock-photo-browse-size").setAttribute('thumb', 'true');
        this.mSize = 'thumb';
    }
    flock.prefs.setCharPref("flock.photo.photoSize", this.mSize);
    this.refreshView();
}

photoBrowser.onFindUserChange = function(aEvent) {
    if(aEvent.keyCode==aEvent.DOM_VK_RETURN) {
        this.findUser();
    }
}

photoBrowser.slideyBoxListener = {
    onDrop: function(aSelection, aHoverChild) {
    },
    onScrollLeft: function() {
    },
    onScrollRight: function() {
        var pagesLeft = $("flock-photo-browse-slideybox").pagesLeft;
        if(pagesLeft<3.3) {
            photoBrowser.mCurrentSearch.getMorePhotos();
        }
    },
    onRemoveSelectionLeft: function() {
    },
    onRemoveSelection: function(aSelection) {
    }
}

// there is a base class implicit here that needs to be codified
function PhotoBrowserNewPhotosGetter (aPeopleList, aTerm) {
    this.mPeopleList = aPeopleList;
    this.mOriginalCount = aPeopleList.length;
    this.mTerm = aTerm;
    if(!this.mPeopleList.length) {
        photoBrowser.setStatus($("slideyMessages").getAttribute("noImagesFound"));
    }
}

PhotoBrowserNewPhotosGetter.prototype = {
    invalid: false,
    mPeopleList: null,
    mTerm: null,
    mOriginalCount: null,
    mFailCount: 0,
    mCurrentSearch: null,
    onSearchResult: function() {// we can't proxy through the enumerator any more
        if(this.invalid) return;
        this.getMorePhotos();
        photoBrowser.PPSVC.clearHasNewPhotos(null);
    },
    onError: function(aError) {
        if(this.invalid) return;
        if(++this.mFailCount==2) return;//this is fairly arbitrary error handling
        this.getMorePhotos();
    },
    getMorePhotos: function() {
        if(this.invalid) return;
        if(this.mPeopleList.length==0) return; 
        if(this.mCurrentSearch) {
            this.mCurrentSearch.cancel();
        }
        //this is a different nCurrentSearch than photoBrowser.mCurrentSearch,
        //which is prolly "this"
        photoBrowser.mCurrentPerson = null;
        var person = this.mPeopleList.shift();
        this.mCurrentSearch = new PhotoBrowserPhotosGetter(person,$('flock-photo-searchbox').value);
        this.mCurrentSearch.messagesDisabled = true;
        this.mCurrentSearch.mListener = this;
        this.mCurrentSearch.mOnlyRecent = true;
        this.mCurrentSearch.getMorePhotos();
    },
    cancel: function() {
        if(this.mCurrentSearch) {
            this.mCurrentSearch.cancel();
        }
        this.invalid = true;
    },
}

function PhotoBrowserPhotosGetter (aPerson, aTerm, aAlbumName) {
    if(!aPerson) throw new Error("need a person silly");
    this.person = aPerson;
    this.term = aTerm;
    this.page = 1;
    this.inProgress = false;
    this.invalid = false;
    this.albumName = aAlbumName;
    this.inst = photoBrowser;
    this.init();
}

PhotoBrowserPhotosGetter.prototype = {
    init: function() {
        photoBrowser.setStatus(null);
    },
    // this listener is just anotherdump('>>> the searchMode is ' + searchMode + ' and the text is ' + searchString + '\n'); search result listener that may be
    // interested in knowing when calls are finished
    mListener: null,
    mOnlyRecent: false,
    setListener: function(aListener) {
        this.mListener = aListener;
    },
    onSearchResult: function(aEnumerator) {
        if(this.invalid) return;
        this.inProgress = false;
        var count = 0;
        while(aEnumerator.hasMoreElements()) {
            var photo  = aEnumerator.getNext();
            photo.QueryInterface(Components.interfaces.flockIPhoto);
            var seq = parseFloat(photo.id);
            if((!this.mOnlyRecent) || (this.person.lastNewSeq <= seq)) {
                //dump((this.person.lastNewSeq < seq) + " " + !this.mOnlyRecent + " " + this.person.lastNewSeq + " " + seq + "\n");
                photoBrowser.appendPhoto(photo);
                ++count;
            }
        }
        if(!this.messagesDisabled) {
            if(this.page==1 && count==0) {
                //if this is my account and i have no pictures - HELP ME WITH
                //INSTRUCTIONS
                if(photoBrowser.PPSVC.isInList(LIST_ACCOUNTS, photoBrowser.mCurrentPerson) && $('flock-photo-searchbox').value=="") {
                    photoBrowser.setStatus($("slideyMessages").getAttribute("noMyImagesFound"));
                }
                else {
                    photoBrowser.setStatus($("slideyMessages").getAttribute("noImagesFound"));
                }
                $('flock-photo-searchbox').setAttribute("style", "color: red;font-weight: bolder;");
            }
            else {
                $('flock-photo-searchbox').removeAttribute("style");
                photoBrowser.setStatus(null);
            }
        }
        if(photoBrowser.mCurrentPerson==this.person) {//note that this currently will not clear for new photos!
            photoBrowser.PPSVC.clearHasNewPhotos(this.person);
        }
        photoBrowser.setupMenulists();
        ++this.page;
        if(this.mListener) this.mListener.onSearchResult();
    },
    onError: function(aError) {
        if(this.invalid) return;
        this.inProgress = false;
        if(!this.messagesDisabled) {
            //photoBrowser.setStatus($("slideyMessages").getAttribute("unableToRetrieve"));
            photoBrowser.setStatus(aError.errorString);
        }
        if(this.mListener) this.mListener.onError(aError);
    },
    cancel: function() {
        this.invalid = true;
    },
    getMorePhotos: function() {
        if(this.invalid) return;
        if(this.inProgress) return;
        this.inProgress = true;
        var tag = null;
        var text = null;
        if(!this.messagesDisabled) {
            if(this.page==1) photoBrowser.setStatus($("slideyMessages").getAttribute("searching"));
        }

        var tag,text;
        var searchString = $('flock-photo-searchbox').value;           
	var tagSearch = searchString.match(/^tags?\s*:\s*(.*?)\s*$/);
	if (tagSearch) {
	    tag = searchString.match(/^tags?\s*:\s*(.*?)\s*$/)[1];
	} else {
	    text = searchString;
	}
        this.person.service.search(this, this.person.id, tag, text, 50, this.page, this.albumName);
        photoBrowser.refreshSlideyboxSize();
    },
}
/////////////////////////////
// functions
/////////////////////////////
photoBrowser.setStatus = function(aMsg) {
    if(!aMsg || aMsg=='') {
        $("flock-photo-browse-status").setAttribute("collapsed", true);
    }
    else {
        $("flock-photo-browse-status").removeAttribute("collapsed");
        $("flock-photo-browse-status").value = aMsg;
    }
}

photoBrowser.appendPhoto = function(aPhoto) {
    var newPhoto = document.createElement("photo");
    $("flock-photo-browse-slideybox").appendChild(newPhoto);
    newPhoto.size = this.mSize;
    newPhoto.photo = aPhoto;
}

photoBrowser.thereIsANewQuery = function() {
    if(!this.mCurrentSearch) return true;
    return true; 
}

photoBrowser.sizeTopbar = function() {
    var topbar = $$('topbarBox');
    var slidey = $('flock-photo-browse-slideybox');
    if(topbar) {
        switch(this.mSize) {
            case "thumb" :
                topbar.setAttribute("height", 130);
            break;
            case "medium" :
                topbar.setAttribute("height", 205);
            break;
        }
    }

    switch(this.mSize) {
        case "thumb" :
            slidey.height = 95;
            slidey.removeAttribute("photoSizeLarge");
        break;
        case "medium" :
            slidey.height = 200;
            slidey.setAttribute("photoSizeLarge", true);
        break;
    }

}

photoBrowser.refreshSlideyboxSize = function() {
    $("flock-photo-browse-slideybox").setAttribute("width", $("photoBrowser").boxObject.width - 10);//beware of this constant 10!
}

photoBrowser.clearSlideyBox = function() {
    while($("flock-photo-browse-slideybox").childNodes.length>1) {
        $("flock-photo-browse-slideybox").removeChild($("flock-photo-browse-slideybox").lastChild);
    }
}

photoBrowser.loadAllNewPre = function() {
    if(this.mCurrentPerson!=null) {
        //search clearing behavior - could be cleared up
        $("flock-photo-searchbox").value="";
    }
    this.mViewType = VIEW_MULTI;
    this.clearSlideyBox();
    this.clearPerson();
    this.sizeTopbar();
    photoBrowser.setStatus($("slideyMessages").getAttribute("retrieving"));
    $("flock-photo-browse-user").removeAttribute("hidden");
    $("flock-photo-browse-user").label = $("flock-photo-browse-user").getAttribute("newPhotosMessage");
    $("flock-photo-browse-user").image = 'chrome://browser/skin/flock/photo/new.png';
    $("cmd_photoPersonToggleWatched").setAttribute("disabled", true);
    window.setTimeout("photoBrowser.loadAllNew()",0);
}

photoBrowser.loadAllNew = function() {
    this.doSearch();
}

photoBrowser.loadPersonPre = function(aPerson) {
    if(!this.mCurrentPerson || this.mCurrentPerson.id!=aPerson.id) {
        //search clearing behavior
        $("flock-photo-searchbox").value="";
    }
    this.mViewType = VIEW_SINGLE;
    this.mCurrentPerson = aPerson;
    this.clearSlideyBox();
    this.updateUIWithNewPerson();
    photoBrowser.setStatus($("slideyMessages").getAttribute("retrieving"));
    $("cmd_photoPersonToggleWatched").setAttribute("disabled", false);
    window.setTimeout("photoBrowser.loadPerson()",0);
}

photoBrowser.loadPerson = function(aPerson) {
    this.hideFindUserDlg();
    if(!aPerson) aPerson = this.mCurrentPerson;
    while($("flock-photo-browse-slideybox").childNodes.length>1) {
        $("flock-photo-browse-slideybox").removeChild($("flock-photo-browse-slideybox").lastChild);
    }
    this.mCurrentPerson = aPerson;
    this.mLastLoad = new Date().getTime();
    this.updateUIWithNewPerson();
    this.doSearch();
    this.PPSVC.addToRecent(this.mCurrentPerson);
    $("flock-photo-browse-slideybox").scrollHome();
    this.setupAlbumMenu();
    this.sizeTopbar();
    var filterText='';
    var bundle = document.getElementById('photoBundle');
    if (this.mCurrentPerson.service.supportsFeature('tags'))
        filterText = bundle.getString("flock.photo.service.filter1");
    else
        filterText = bundle.getString("flock.photo.service.filter2");

     $('flock-photo-searchbox').setAttribute('tooltiptext', filterText);
     $('flock-photo-browse-search-image').setAttribute('tooltiptext', filterText);
}

photoBrowser.getNewList = function() {
    var rval = new Array();
    var kids = $("flock-photo-browse-new-menu").childNodes;
    for(var i=0;i<kids.length;++i) {
        var child = kids.item(i);
        if(child.nodeName!="menuitem") continue;
        if((child.getAttribute("hasNew")=="true")) {
            rval.push(photoBrowser.PPSVC.getPhotoPersonByURI(child.id));
        }
    }
    return rval;
}

photoBrowser.doSearch = function(aAlbumName) {
    if(this.thereIsANewQuery()) {
        if(this.mCurrentSearch) {
            this.mCurrentSearch.cancel();
        }
        switch(this.mViewType) {
            case VIEW_MULTI:
                var list = this.getNewList();
                this.mCurrentSearch = new PhotoBrowserNewPhotosGetter(list,$('flock-photo-searchbox').value);
                this.mCurrentSearch.getMorePhotos();
            break;
            case VIEW_SINGLE:
                this.mCurrentSearch = new PhotoBrowserPhotosGetter(this.mCurrentPerson,$('flock-photo-searchbox').value, aAlbumName);
                this.mCurrentSearch.getMorePhotos();
            break;
        }
    }
}

photoBrowser.browseAlbum = function(eventTarget) {
    if (eventTarget.target.localName != 'menuitem')
        return;
    var selectedAlbum = eventTarget.target.getAttribute('value');
    var albumLabel;
    if (selectedAlbum) {
        switch (selectedAlbum.split('/').length) {   
            case 1:
                albumLabel = eventTarget.target.getAttribute('label').split('/').pop();
            break;        
            default:
                albumLabel = "... > " + eventTarget.target.getAttribute('label').split('/').pop();
            break;
        }
    }
    if (!albumLabel || !albumLabel.length) albumLabel = "";     
    $('flock-photo-browse-user-album').setAttribute('label', albumLabel);
    this.clearSlideyBox();
    $("flock-photo-browse-slideybox").scrollHome();
    this.doSearch(selectedAlbum);
}

photoBrowser.setupAlbumMenu = function() {
    var person = this.mCurrentPerson;
    if (!person) {
        throw new Error('No person to get album info from!')
        return;
    }
    
    var albumsMenu = {
        popup: $('flock-photo-browser-albumlist'),
        button: $('flock-photo-browse-user-album'),
	numAlbums: 0,
        
        init : function() {
            this.button.setAttribute('hidden', true);
            this.popup.setAttribute('disabled', true);
            numItems = this.popup.childNodes.length;

            for (var i=0; i < numItems; i++) {
                if (!this.popup.lastChild.hasAttribute('dontDelete'))
                    this.popup.removeChild(this.popup.lastChild);
            }
        },
        
        refresh: function(albumEnumerator) {
            this.init();
            var albumId;
            var albumTitle;
            this.numAlbums = 0;
            while(albumEnumerator.hasMoreElements()) {
                var album = albumEnumerator.getNext();
                album.QueryInterface(Components.interfaces.flockIPhotoAlbum);
                var menuitem = document.createElement("menuitem");
                this.popup.appendChild(menuitem);
                menuitem.setAttribute("value", album.id);
                menuitem.setAttribute("label", album.title);
                this.numAlbums++;
            }
            this.popup.removeAttribute('disabled');
            albumsMenu.button.setAttribute('hidden', this.numAlbums==0);
        },
    }
    
    var albumsGetterListener = {
       onGetAlbumsResult: function(aEnumerator) {
           albumsMenu.refresh(aEnumerator);
     },
      
      onError: function(aError) {
          dump('albumsGetterListener error - ' + aError.errorString + '\n');
      }
    }
   
    var currAlbumToolbarBtn = $('flock-photo-browse-user-album');
    currAlbumToolbarBtn.setAttribute('label', '');
    albumsMenu.init();
    
    if (person.id.length )
        person.service.getAlbums(albumsGetterListener, person.id);
    else
      albumsMenu.button.setAttribute('hidden', true);
}

photoBrowser.updateUIWithNewPerson = function() {
    $("flock-photo-browse-user").removeAttribute("hidden");

    var serviceName = this.mCurrentPerson.service.serviceName;
    dump('UPDATE FAVIOCN FOR SERVICE!!!');

    $("flock-photo-browse-user").label = this.mCurrentPerson.username;
    $("flock-photo-browse-user").image = this.mCurrentPerson.service.iconUrl;
    $("flock-photo-browse-service-menu").value = this.mCurrentPerson.service.shortName;
    $("flock-photo-browse-username").value = this.mCurrentPerson.username;
    if(this.PPSVC.isInList(LIST_WATCHED, this.mCurrentPerson)) {
        $("cmd_photoPersonToggleWatched").setAttribute("watch", "true");
        $("cmd_photoPersonToggleWatched").setAttribute("label", $("cmd_photoPersonToggleWatched").getAttribute("unwatchText"));
    }
    else {
        $("cmd_photoPersonToggleWatched").setAttribute("watch", "false");
        $("cmd_photoPersonToggleWatched").setAttribute("label", $("cmd_photoPersonToggleWatched").getAttribute("watchText"));
    }
}

photoBrowser.clearPerson = function() {
    this.mCurrentPerson = null;
    $("flock-photo-browse-user").setAttribute("hidden", true);
}

photoBrowser.showFindUserDlg = function() {
    photoBrowser.initFindUserUI();
    $("flock-photo-browse-slideybox-deck").selectedIndex = 1;
    $("flock-photo-browse-tab-deck").selectedIndex = 1;
    if(!this.mCurrentPerson || this.mCurrentPerson.id=="") $("flock-photo-browse-username").value = "";
    $("flock-photo-browse-username").select();
    $("flock-photo-browse-find-user-msg").value = "";
    return;
}

photoBrowser.hideFindUserDlg = function(aCancelCurrentRequest) {
    $("flock-photo-browse-slideybox-deck").selectedIndex = 0;
    $("flock-photo-browse-tab-deck").selectedIndex = 0;
}

photoBrowser.mLastFindListener = null;

photoBrowser.findListener = function() {
}

photoBrowser.findListener.prototype = {
    invalid: false,
    onFindByUsernameResult: function(aPerson) {
        if(this.invalid) return;
        photoBrowser.loadPersonPre(aPerson);
        photoBrowser.hideFindUserDlg();
    },
    onError: function(aError) {
        if(this.invalid) return;
        //$("flock-photo-browse-find-user-msg").value = $("flock-photo-browse-find-user-msg").getAttribute("errorText");
        $("flock-photo-browse-find-user-msg").value = aError.errorString;
        $("flock-photo-browse-username").select();
    },
}

photoBrowser.findUser = function() {
    var service = photoBrowser.APIMGR.getAPIFromShortname($("flock-photo-browse-service-menu").value);
    if(this.mLastFindListener) this.mLastFindListener.invalid = true;
    this.mLastFindListener = new photoBrowser.findListener();
    service.findByUsername(this.mLastFindListener, $("flock-photo-browse-username").value.replace(/^\s+/,'').replace(/\s+$/,'') );
    $("flock-photo-browse-find-user-msg").value = $("flock-photo-browse-find-user-msg").getAttribute("searchingText");
}

photoBrowser.toggleWatched = function(aPerson) {
    var person = (aPerson?aPerson:this.mCurrentPerson);
    if(this.PPSVC.isInList(LIST_WATCHED, person)) {
        this.PPSVC.removeFromList(LIST_WATCHED, person);
    }
    else {
        this.PPSVC.addToList(LIST_WATCHED, person);
        // adding the following hack to force sorting - will investigate in a
        // bit
        this.PPSVC.removeFromList(LIST_WATCHED, person);
        this.PPSVC.addToList(LIST_WATCHED, person);
    }
    this.updateUIWithNewPerson();
    photoBrowser.setupMenulists();
}

photoBrowser.onkeypress = function(aEvent, aElement) {
    switch(aEvent.keyCode) {
        case aEvent.DOM_VK_RIGHT:
            $("flock-photo-browse-slideybox").scrollRight();
            return
        break;
        case aEvent.DOM_VK_LEFT:
            $("flock-photo-browse-slideybox").scrollLeft();
        break;
    }
}

photoBrowser.lookForNewPhotos = function(aListid) {
    photoBrowser.PPSVC.findNewPhotosForList(aListid);
}
 
photoBrowser.toggleTagSearchbox = function() {
    var elm = $('flock-photo-browser-search-container');
    if (elm.hasAttribute('open')) {
        elm.removeAttribute('open');
        $('flock-photo-searchbox').setAttribute('hidden', true); 
        $('flock-photo-searchbox').value = '';
        this.refreshView();
    } else {
        $('flock-photo-searchbox').removeAttribute('hidden');
        elm.setAttribute('open', true);
        $('flock-photo-searchbox').focus();
    }
    
}

photoBrowser.updateLoginUI = function (aAPI) {
    var loginButton = $('flock-photo-browser-login');
    switch (aAPI.authState) {
        case aAPI.LOGGED_IN:
            loginButton.hidden = true;
            loginButton.disabled = false;   
        break;
        case aAPI.LOGGED_OUT:
            loginButton.hidden = false; 
            loginButton.setAttribute('label', 'Login')
            loginButton.disabled = false;
        break;
        
        case aAPI.LOGGING_IN:
            loginButton.setAttribute('label', 'Logging in...')
            loginButton.disabled = true;
        break;
        
        default:
        break;      
    }
}

photoBrowser.login = function() {
    var defaultService = photoBrowser.APIMGR.getDefaultService();
    if (defaultService) {
        photoBrowser.APIMGR.doLogin(defaultService.shortName)
    } else {
        flock.photo.util.launchSettings();
    }
    
}
