// vim: ts=4 sw=4 cindent expandtab
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL
//
var panePhotos = {};
const PREF_TAGLINE = 'flock.photo.tagline';


panePhotos.unload = function() {
    panePhotos.mApiMgr.removeListener(panePhotos.mListener);
    panePhotos.mPrefObserverSvc.removeObserver (PREF_TAGLINE, panePhotos);
}

panePhotos.load = function() {
    
    // removing the spread flock photo feature as it is half backed
    /*
    
    if (document.getElementById('flock.photo.spreadFlock.is_disabled').value) {
        document.getElementById('spread-flock-box').hidden=true;
    }
    
    */
    
    this.mApiMgr = Components.classes['@flock.com/photo-api-manager;1?']
        .getService(Components.interfaces.flockIPhotoAPIManager);

    this.mPrefSvc = Components.classes["@mozilla.org/preferences-service;1"]
                               .getService(Components.interfaces.nsIPrefBranch);
    this.mPrefObserverSvc = this.mPrefSvc.QueryInterface (Components.interfaces.nsIPrefBranchInternal);

    this.setupList();
    this.updateCommands();
    this.mListener = {
        onAPIStateChange: function(aAPI) {
            panePhotos.updateCommands();
        },
        onError: function(aError) {
            alert(aError.errorString);
        },
    }

    this.setupImage ();
    this.mPrefObserverSvc.addObserver (PREF_TAGLINE, this, false);

    this.mApiMgr.addListener(this.mListener);
    window.addEventListener('unload', panePhotos.unload, false);
}

panePhotos.getServices = function(aAll) {
    var rval = {};
    if(aAll) {
        var services = this.mApiMgr.services;
    }
    else {
        var services = this.mApiMgr.preferredServices;
    }
    while(services.hasMoreElements()) {
        var svc = services.getNext();
        rval[svc.shortName] = svc;
    }
    return rval;
}

panePhotos.additionalServices = function() {
    window.openDialog("chrome://browser/content/flock/preferences/allPhotoSvcs.xul", "Enable Additional Photo Services", "modal, centerscreen");
}

panePhotos.setupList = function() {
    var preferredServices = this.getServices(true);
    var listing = $("listing");
    
    while(listing.childNodes.length) {
        try {
            listing.lastChild.removeListener();
        } catch(e) { }
        listing.removeChild(listing.lastChild);
    }

    for(var p in preferredServices) {
        var svc = preferredServices[p];
        var serviceName = svc.serviceName; if(!serviceName) continue;//tmp workaround
        var account = document.createElement("photoAccount");
        account.service = svc;
        listing.appendChild(account);
    }
    
    if (!listing.childNodes.length) {
        var noAccountMsg = document.createElement('label');
        noAccountMsg.setAttribute('value', "No photo service accounts setup.");
        listing.appendChild(noAccountMsg);
    }
}

panePhotos.addPhotoAccount = function() {
}

panePhotos.doCommand = function(aID) {
    var cmd = panePhotos[aID];
    cmd.exec();
    panePhotos.updateCommands();
}

panePhotos.updateCommands = function() {
    var set = $("photoPrefCommands");
    for(var i=0;i<set.childNodes.length;++i) {
        var element = set.childNodes.item(i);
        var cmd = panePhotos[element.getAttribute("id")];
        if(cmd.isEnabled()) {
            element.removeAttribute("disabled");
        }
        else {
            element.setAttribute("disabled", true);
        }
    }
}

panePhotos.cmd_addPhotoAccount = {
    isEnabled: function() {
        var services = panePhotos.getServices(true);
        var preferredServices = panePhotos.getServices();
        var a=0;
        var b=0;
        for(var s in services) ++a;
        for(var s in preferredServices) ++b;
        if(a==b) return false;
        return true;
    },
    exec: function() {
        window.openDialog("chrome://browser/content/flock/photo/photoAccountWizard.xul", "PhotoAccountWizard", "modal, centerscreen");
        panePhotos.setupList();
    },
}

panePhotos.launchTaglineComposer = function() {
    window.openDialog ("chrome://browser/content/flock/photo/taglineChooser.xul", "_blank", "modal,close,centerscreen");
}

panePhotos.setupImage = function () {
    document.getElementById('flock-props-image').setAttribute ('src', 'http://spread.flock.com/taglines/images/' +
    this.mPrefSvc.getCharPref (PREF_TAGLINE));
}

panePhotos.observe = function (subject, topic, prefName) {
    if (topic == 'nsPref:changed' && prefName == PREF_TAGLINE) {
        this.setupImage ();
    }
}
